package net.pijcke.mazes.generation

import net.pijcke.mazes.rectangular.RectangularGrid
import net.pijcke.mazes.rectangular.CartesianPosition
import net.pijcke.mazes.rectangular.RectangularGridWalker
import java.util.Random

object SideWinder {

    fun on(grid: RectangularGrid, walker: RectangularGridWalker) {
        val expandablePositions = mutableListOf<CartesianPosition>()

        walker.makeSequence(grid).forEach { currentPosition ->
            // Connect top row
            if (!grid.isWithinBounds(currentPosition.north())) {
                if (grid.isWithinBounds(currentPosition.east())) {
                    connectEast(grid, currentPosition)
                }
                return@forEach
            }

            expandablePositions.add(currentPosition)
            if (grid.isWithinBounds(currentPosition.east()) && Random().nextBoolean()) {
                connectEast(grid, currentPosition)
            }
            else {
                connectCluster(grid, expandablePositions)
            }
        }
    }

    private fun connectEast(grid: RectangularGrid, position: CartesianPosition) {
        println("Connecting $position east.")
        grid.link(position, position.east())
    }

    private fun connectCluster(grid: RectangularGrid, expandablePositions: MutableList<CartesianPosition>) {
        val positionToExpand = expandablePositions.random()
        println("Connecting cluster north : $expandablePositions. Picked $positionToExpand")
        grid.link(positionToExpand, positionToExpand.north())
        expandablePositions.clear()
    }

}