package net.pijcke.mazes.grid

interface GridWalker<P: Position<P>> {

    fun origin(grid: Grid<P>): P

    fun unitVectors(): List<(P) -> P>

    fun makeSequence(grid: Grid<P>): Sequence<P> = makeSequence(grid, origin(grid), unitVectors())

    private fun makeSequence(grid: Grid<P>, startingPoint: P, remainingUnitVectors: List<(P) -> P>): Sequence<P> {
        if (remainingUnitVectors.isEmpty()) {
            return sequenceOf(startingPoint)
        }

        val curF = remainingUnitVectors.first()
        val nextVectors = remainingUnitVectors.drop(1)
        return generateSequence(startingPoint) { curF(it) }
            .takeWhile { grid.isWithinBounds(it) }
            .flatMap { makeSequence(grid, it, nextVectors) }
    }

}
