package net.pijcke.mazes.grid.cell

import net.pijcke.mazes.grid.Grid
import net.pijcke.mazes.grid.Position
import java.lang.IllegalArgumentException

abstract class CellGrid<P: Position<P>>(cells: Collection<Cell<P>>): Grid<P> {

    private val cells: Map<P, Cell<P>> = cells.associateBy { it.position }

    override fun getPositions() = cells.keys

    override fun link(position1: P, position2: P) {
        val cell1 = cells[position1] ?: throw IllegalArgumentException("No cell at position $position1.")
        val cell2 = cells[position2] ?: throw IllegalArgumentException("No cell at position $position2.")
        cell1.link(cell2)
        cell2.link(cell1)
    }

    override fun linked(position1: P, position2: P): Boolean {
        return cells[position1]?.isLinkedTo(position2) ?: false
    }

}